%To simulate a Smooth Transition Autorregressive Model
%Vivaldo M. Mendes, ISCTE-Lisbon University Institute, January 2009

%Parameters
a0=1.5;
a1=0.5;
a2=2;
a3=0.6;
c=3; % Threshold parameter
gamma=2.5;% Parameter that gives the speed of transition between the two regimes



%initial condition
y(1)=0.5;
h(1)=0.5;
g(1)=0.5;


% Defining white noise

t=200;%number of observations
wn=randn(t,1);


% Consider two independ stochastic processes (h(t) and g(t))
% Loop for each regime separately
n=200; % number of iterations
for t=1:n
    h(t+1)=a0+a1*h(t)+wn(t);
    g(t+1)=a2+a3*g(t)+wn(t);
end

% Defining the Logistic function
% Now consider that just 1 stochastic process which puts the two
% regimes together (y(t)). The process jumps from one regime to the other 
% if the former goes above/below a certain THRESHOLD
% Loop for the 2-Regime AR: threshold is 3.5 in this example

for t=1:n 
    if y(t)<c;
        L(t)=(1+exp(-gamma*(y(t)-c)))^(-1); % Defining the Logistic function
        y(t+1)=(a0+a1*y(t))*(1-L(t))+wn(t);
    else
        L(t)=(1+exp(-gamma*(y(t)-c)))^(-1);
        y(t+1)=(a2+a3*y(t))*L(t)+wn(t);
    end
   
end

% Exemplify the Logistic Transition function
x=[-4:0.01:4];
v=0;
figure;
hold on
for eta=[1,2,4,100]
    w=(1+exp(-eta.*(x-v))).^(-1);
    plot(x,w)
   end
legend('eta=1','eta=2','eta=4','eta=100')
grid on
xlabel('$x$','fontsize',16,'interpreter','latex'); 
ylabel('$w$','fontsize',16,'interpreter','latex');
title('Logistic function')


% make figure of the Logistic function (L(t))
figure
plot(L);
xlabel('$t$','fontsize',16,'interpreter','latex'); 
ylabel('$L(t)$','fontsize',16,'interpreter','latex');
title('Transition function')
grid on
axis([1 200 0 1]);

% make figure of the Logistic function (L(t))against y(t)
figure
y1=(y(1:end-2));% The last 2 observations of y(t+1)removed (get 200 observ)
L1=(L(1:end-1));% To get also 200 observations
plot(y1,L1,'.');
xlabel('$y(t)$','fontsize',16,'interpreter','latex'); 
ylabel('$L(t)$','fontsize',16,'interpreter','latex');
title('Logistic Transition Function')
grid on

% make figure of the 
figure
plot(y,'b-o');hold on
plot(h,'g'); hold on
plot(g,'r')
axis([1 120 0 8]);
%%%%%%%%%%%% Just fancy editing
legend ('y','h','g')
xlabel('$t$','fontsize',16,'interpreter','latex'); 
ylabel('$g,h,y$','fontsize',16,'interpreter','latex');
grid on
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%

% Plot white noise 
figure
plot(wn,'-r');
title('White Noise');
xlabel('$t$','fontsize',16,'interpreter','latex'); 
ylabel('$wn$','fontsize',16,'interpreter','latex');
grid on;

% Calculate autocorrlation of x, y and wn

Ag=autocorr(g,30);
Ah=autocorr(h,30);
Ay=autocorr(y,30);
Awn=autocorr(wn,30);

% Plot autocorrelation
figure
plot(Ag,'-or');hold on
plot(Ah,'-ok');hold on
plot(Ay,'-ob');hold on
plot(Awn,'-*');
xlabel('$t$','fontsize',16,'interpreter','latex'); 
legend('Ag','Ah','Ay','Awn')
grid on;
title('Autocorrelation')

% Standard deviation and mean

Sg=std(g);
Sh=std(h);
Sy=std(y);
Mg=mean(g);
Mh=mean(h);
My=mean(y);

figure
y2=y(2:end);
z=y(1:end-1);
plot(y2,z,'*')
xlabel('$y(t)$','fontsize',16,'interpreter','latex');
ylabel('$y(t+1)$','fontsize',16,'interpreter','latex');
grid on;
title('Autocorrelation')

