%To simulate a Threshold Autorregressive Model
%Vivaldo M. Mendes, ISCTE-Lisbon University Institute, January 2009

%Parameters
a0=1.5;
a1=0.5;
a2=2;
a3=0.6;
c=3.5;




%initial condition
y(1)=0.5;
h(1)=0.5;
g(1)=0.5;

% Defining white noise

t=200;%number of observations
wn=randn(t,1);


% Consider two independ stochastic processes (h(t) and g(t))
% Loop for each regime separately
n=200; % number of iterations
for t=1:n
    h(t+1)=a0+a1*h(t)+wn(t);
    g(t+1)=a2+a3*g(t)+wn(t);
end

% Now consider that just 1 stochastic process which puts the two
% regimes together (y(t)). The process jumps from one regime to the other 
% if the former goes above/below a certain THRESHOLD
% Loop for the 2-Regime AR: threshold is 3.5 in this example

for t=1:n 
    if y(t)<c
        y(t+1)=a0+a1*y(t)+wn(t);
    else
      y(t+1)=a2+a3*y(t)+wn(t);
    end
   
end

% make figure of the 
figure
plot(y,'b-o');hold on
plot(h,'g'); hold on
plot(g,'r')
axis([1 120 0 8]);
%%%%%%%%%%%% Just fancy editing
legend ('y','h','g')
xlabel('$t$','fontsize',16,'interpreter','latex'); 
ylabel('$g,h,y$','fontsize',16,'interpreter','latex');
grid on
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%

% Plot white noise 
figure
plot(wn,'-r');
title('White Noise');
xlabel('$t$','fontsize',16,'interpreter','latex'); 
ylabel('$wn$','fontsize',16,'interpreter','latex');
grid on;

% Calculate autocorrlation of x, y and wn

Ag=autocorr(g,30);
Ah=autocorr(h,30);
Ay=autocorr(y,30);
Awn=autocorr(wn,30);

% Plot autocorrelation
figure
plot(Ag,'-or');hold on
plot(Ah,'-ok');hold on
plot(Ay,'-ob');hold on
plot(Awn,'-*');
xlabel('$t$','fontsize',16,'interpreter','latex'); 
legend('Ag','Ah','Ay','Awn')
grid on;
title('Autocorrelation')

% Standard deviation and mean
Sg=std(g)
Sh=std(h)
Sy=std(y)
Mg=mean(g)
Mh=mean(h)
My=mean(y)
